import 'package:flutter/material.dart';

import 'ColorDescriptor.dart';

const double _ITEM_HEIGHT = 48;

class ColorContainer extends StatelessWidget {
  const ColorContainer({
    Key? key,
    required this.colorDescriptor,
    required this.isSelected,
    this.onTap,
    this.onDoubleTap,
    this.height = _ITEM_HEIGHT,
  }) : super(key: key);

  final ColorDescriptor colorDescriptor;
  final bool isSelected;
  final ValueSetter<ColorDescriptor>? onTap;
  final ValueSetter<ColorDescriptor>? onDoubleTap;
  final double height;

  String get _colorString => "#${colorDescriptor.color!.value.toRadixString(16).padLeft(8, '0').toUpperCase()}";

  @override
  Widget build(BuildContext context) {
    return GestureDetector(
      onTap: onTap == null ? null : () => onTap!(colorDescriptor),
      onDoubleTap: onDoubleTap == null ? null : () => onDoubleTap!(colorDescriptor),
      child: Semantics(
        container: true,
        child: Container(
          height: height,
          padding: const EdgeInsets.symmetric(horizontal: 16),
          color: colorDescriptor.color,
          child: Row(
            mainAxisAlignment: MainAxisAlignment.spaceBetween,
            crossAxisAlignment: CrossAxisAlignment.center,
            children: [
              Text(colorDescriptor.getTitle()),
              if (isSelected)
                Icon(
                  Icons.check,
                  color: colorDescriptor.contrastColor,
                ),
              Flexible(child: Text(_colorString)),
            ],
          ),
        ),
      ),
    );
  }
}
