import 'package:flutter/material.dart';

import 'ColorDescriptor.dart';
import 'ColorTable.dart';

class ColorPalette {
  static const primaryKeys = <int>[50, 100, 200, 300, 400, 500, 600, 700, 800, 900];
  static const accentKeys = <int>[100, 200, 400, 700];

  ColorPalette({
    required this.name,
    required this.primary,
    this.accent,
    this.threshold = 900,
    this.singleColor = false,
  }) {
    variants = _generateVariants();
  }

  final bool singleColor;
  final String name;
  final Color primary;
  final MaterialAccentColor? accent;
  final int threshold; // Contrast color for indices > threshold are white, otherwise black.
  late final List<ColorDescriptor> variants;

  ColorDescriptor toColorDescriptor() {
    return ColorDescriptor(
      color: primary,
      prefix: accent == null ? '' : 'A',
    );
  }

  List<ColorDescriptor> getVariants({bool withBaseColor = false}) {
    if (withBaseColor) {
      return [toColorDescriptor(), ...variants];
    } else {
      return variants;
    }
  }

  Color getPrimary(int? key) {
    if (primary is MaterialColor) {
      return (primary as MaterialColor)[key!]!;
    } else {
      return primary;
    }
  }

  List<ColorDescriptor> _generateVariants() {
    final result = <ColorDescriptor>[];

    if (singleColor) {
      if (name == colorsBlack) {
        result.addAll([
          ColorDescriptor(color: Colors.black, contrastColor: Colors.white),
          ColorDescriptor(name: 'black87', color: Colors.black87, contrastColor: Colors.white),
          ColorDescriptor(name: 'black54', color: Colors.black54, contrastColor: Colors.white),
          ColorDescriptor(name: 'black45', color: Colors.black45, contrastColor: Colors.white),
          ColorDescriptor(name: 'black38', color: Colors.black38, contrastColor: Colors.white),
          ColorDescriptor(name: 'black26', color: Colors.black26, contrastColor: Colors.white),
          ColorDescriptor(name: 'black12', color: Colors.black12, contrastColor: Colors.white),
          ColorDescriptor(color: Colors.white, contrastColor: Colors.black),
        ]);
      } else {
        result.addAll([
          ColorDescriptor(color: Colors.white, contrastColor: Colors.black),
          ColorDescriptor(name: 'white70', color: Colors.white70, contrastColor: Colors.black),
          ColorDescriptor(name: 'white60', color: Colors.white60, contrastColor: Colors.black),
          ColorDescriptor(name: 'white54', color: Colors.white54, contrastColor: Colors.black),
          ColorDescriptor(name: 'white38', color: Colors.white38, contrastColor: Colors.black),
          ColorDescriptor(name: 'white30', color: Colors.white30, contrastColor: Colors.black),
          ColorDescriptor(name: 'white24', color: Colors.white24, contrastColor: Colors.white),
          ColorDescriptor(name: 'white12', color: Colors.white12, contrastColor: Colors.white),
          ColorDescriptor(name: 'white10', color: Colors.white10, contrastColor: Colors.white),
          ColorDescriptor(color: Colors.black, contrastColor: Colors.white),
        ]);
      }
    } else {
      // Primary colors
      for (final key in primaryKeys) {
        final contrastColor = key > threshold ? Colors.white : Colors.black;
        result.add(
          ColorDescriptor(shade: key, color: getPrimary(key), contrastColor: contrastColor),
        );
      }
      // Accent colors
      if (accent != null) {
        for (final key in accentKeys) {
          final contrastColor = key > threshold ? Colors.white : Colors.black;
          result.add(
            ColorDescriptor(shade: key, color: accent![key], contrastColor: contrastColor, prefix: 'A'),
          );
        }
      }
    }

    return result;
  }
}
