import 'package:dotup_flutter_widgets/scroll_wheel/ScrollWheelTimeSelector.dart';
import 'package:dotup_flutter_widgets/scroll_wheel/scroll_wheel.dart';
import 'package:flutter/material.dart';

class SectionTile extends StatelessWidget {
  final String? text;
  final Widget? icon;
  final VoidCallback? onTap;
  final Color color;
  final Widget? subTitle;
  final Widget? title;
  final Widget? trailing;

  const SectionTile({
    Key? key,
    this.text,
    this.icon,
    this.onTap,
    required this.color,
    this.title,
    this.subTitle,
    this.trailing,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return ListTile(
      contentPadding: EdgeInsets.symmetric(horizontal: 10),
      leading: icon,
      title: title ??
          Text(
            text!,
            style: TextStyle(color: color),
          ),
      subtitle: subTitle,
      trailing: trailing,
      onTap: onTap,
      enabled: onTap != null,
    );
  }

  factory SectionTile.navigator({
    Key? key,
    required String text,
    Icon? icon,
    required VoidCallback onTap,
    required Color color,
    String? subTitle,
  }) {
    return SectionTile(
      text: text,
      color: color,
      trailing: Icon(
        Icons.keyboard_arrow_right,
        color: color,
      ),
      onTap: onTap,
    );
  }

  factory SectionTile.button({
    Key? key,
    required String text,
    Icon? icon,
    required VoidCallback onTap,
    required Color color,
    String? subTitle,
  }) {
    final textWidget = Text(
      text,
      style: TextStyle(color: color),
    );

    final title = icon == null
        ? textWidget
        : Row(
            mainAxisAlignment: MainAxisAlignment.center,
            children: [
              icon,
              textWidget,
            ],
          );

    return SectionTile(
      title: Center(
        child: title,
      ),
      color: color,
      onTap: onTap,
    );
  }

  factory SectionTile.switchButton({
    Key? key,
    required String text,
    Icon? icon,
    required ValueSetter<bool>? onChanged,
    required Color color,
    Color? activeColor,
    String? subTitle,
    required bool switchButtonValue,
  }) {
    return SectionTile(
      text: text,
      color: color,
      trailing: Switch(
        materialTapTargetSize: MaterialTapTargetSize.shrinkWrap,
        value: switchButtonValue,
        onChanged: onChanged,
        activeColor: activeColor,
      ),
      onTap: onChanged != null
          ? () {
              onChanged(!switchButtonValue);
            }
          : null,
    );
  }

  factory SectionTile.checkbox({
    Key? key,
    required String text,
    String? subTitle,
    Icon? icon,
    required ValueChanged<bool?>? onChanged,
    required Color color,
    Color? activeColor,
    required bool value,
  }) {
    return SectionTile(
      text: text,
      subTitle: subTitle == null ? null : Text(subTitle),
      color: color,
      trailing: Checkbox(
        value: value,
        onChanged: onChanged,
        activeColor: activeColor,
        checkColor: activeColor,
        materialTapTargetSize: MaterialTapTargetSize.shrinkWrap,
        tristate: false,
      ),
      onTap: onChanged != null
          ? () {
              onChanged(!value);
            }
          : null,
    );
  }

  factory SectionTile.scrollWheelTimeSelector({
    Key? key,
    required String text,
    String? subTitle,
    Icon? icon,
    required ValueChanged<TimeOfDay?> onChanged,
    required Color color,
  }) {
    return SectionTile(
      text: text,
      subTitle: subTitle == null ? null : Text(subTitle),
      color: color,
      trailing: Container(
        width: 110,
        child: ScrollWheelTimeSelector(
          color: color,
          height: 100,
          onTimeChanged: onChanged,
          controller: ScrollWheelTimeSelectorController(hasError: false),
        ),
      ),
      // onTap: onChanged != null
      //     ? () {
      //         onChanged(!value);
      //       }
      //     : null,
    );
  }
}
